"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const helper_module_imports_1 = require("@babel/helper-module-imports");
const helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
const plugin_syntax_jsx_1 = __importDefault(require("@babel/plugin-syntax-jsx"));
const assert_1 = __importDefault(require("assert"));
exports.default = (0, helper_plugin_utils_1.declare)(({ types: t }, { importSources = ["three"] }) => {
    (0, assert_1.default)(Array.isArray(importSources), "importSources must be an array");
    const NS = importSources.map((id) => require(id));
    return {
        inherits: plugin_syntax_jsx_1.default,
        pre() {
            this.imports = new Set();
        },
        post() {
            var _a;
            (_a = this.imports) === null || _a === void 0 ? void 0 : _a.clear();
        },
        visitor: {
            JSXOpeningElement(path) {
                var _a, _b;
                const { name } = path.node;
                if (t.isJSXIdentifier(name)) {
                    const elementName = name.name;
                    const pascalCaseName = elementName.charAt(0).toUpperCase() + elementName.slice(1);
                    for (let i = 0; i < NS.length; i++) {
                        if (pascalCaseName in NS[i]) {
                            (_a = this.imports) === null || _a === void 0 ? void 0 : _a.add(`${importSources[i]},${pascalCaseName}`);
                            break;
                        }
                    }
                }
                else if (t.isJSXMemberExpression(name)) {
                    const propertyName = name.property.name;
                    const pascalCaseName = propertyName.charAt(0).toUpperCase() + propertyName.slice(1);
                    for (let i = 0; i < NS.length; i++) {
                        if (pascalCaseName in NS[i]) {
                            (_b = this.imports) === null || _b === void 0 ? void 0 : _b.add(`${importSources[i]},${pascalCaseName}`);
                            break;
                        }
                    }
                }
                else {
                    throw new Error("Unsupported JSX element name");
                }
            },
            Program: {
                exit(path) {
                    // Add extend import
                    const extendName = (0, helper_module_imports_1.addNamed)(path, "extend", "@react-three/fiber");
                    // Add imports
                    const importsArray = Array.from(this.imports).map((s) => {
                        const [a, b] = s.split(",");
                        return [a, b];
                    });
                    const importNames = importsArray.map(([module, name], i) => (0, helper_module_imports_1.addNamed)(path, name, module));
                    // Add extend call
                    const objectProperties = importsArray.map(([_, name], i) => t.objectProperty(t.identifier(name), importNames[i], false, true));
                    const objectExpression = t.objectExpression(objectProperties);
                    const extendCall = t.expressionStatement(t.callExpression(extendName, [objectExpression]));
                    const body = path.get("body");
                    if (Array.isArray(body)) {
                        const lastImport = body
                            .filter((p) => p.isImportDeclaration())
                            .pop();
                        if (lastImport)
                            lastImport.insertAfter(extendCall);
                    }
                },
            },
        },
    };
});
